/* Copyright 1989-93 GROUPE BULL -- See license conditions in file COPYRIGHT */

/***************************************************************************
 * khash.c
 *
 * This is a simple generic package for hash tables. According to the
 * mode of the hash table, it will acccept (or not) items with the same
 * key.
 *
 * static:
 *
 * externs:
 * HashtableMake  : make a new hashtable
 * HashtableAdd   : add the given entry in hashtable
 * HashtableFind
 * HashtableFindNext 
 * HashtableMap:
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hash.h"


/*
 * A few std cmp/hash function sets
 */

unsigned long OsHashCString (char *cstr) {
    unsigned long ret = 0;
    unsigned long ctr = 0;

    while (*(char*)cstr) {
        ret ^= *(char*)cstr++ << ctr;
        ctr = (ctr + 1) % sizeof (void *);
    }
    return (ret) ;
}

int OsCmpCString (char *s1, char *s2) {
    return (strcmp (s1, s2)) ;
}


/***************************************************************************
 * HashtableMake: make a new hashtable.
 *
 * RETURN: Hashtable, exit on error
 ***************************************************************************/

Hashtable HashtableMake (size, hash_fct, cmp_fct)
    unsigned int  size ;
    unsigned long (*hash_fct)() ;
    int           (*cmp_fct)() ;

{
    Hashtable hashtable ;

    if ( ( ! ( hashtable = (Hashtable) malloc (sizeof (HashtableRec))))
	|| ( ! (hashtable->slots = (Hashslot *) malloc (sizeof (Hashslot *) * size))))
	
    {
	perror ("HashtableMake: malloc ") ;
	exit (1) ;
    }

    memset ((char *) hashtable->slots, 0, sizeof (Hashslot *) * size) ;
    hashtable->size         = size ;
    hashtable->item_count   = 0 ;
    hashtable->hash_fct     = hash_fct ;
    hashtable->cmp_fct      = cmp_fct ;
    hashtable->mode         = 0 ;
    hashtable->current_slot = (Hashslot) 0 ;

    return (hashtable) ;
}

/***************************************************************************
 * HashtableAdd: add the given item to hashtable
 * HashtableRemove: remove given item from hashtable.
 *
 * Slightly depends on the hashtable mode for Add. (Overide)
 * Removes compare item's adresses, not items key.
 *
 * RETURN: a pointer to the item (might != arg, because already in)
 ***************************************************************************/

void *HashtableAdd (hashtable, item)
    register Hashtable hashtable ;
    void               *item ;

{
    register Hashslot slot ;
    Hashslot          *position ;

/*
 * Get the correct slot, check for present :
 */
    position = &(hashtable->slots[(hashtable->hash_fct)(item) % hashtable->size]) ;
    slot     = *position ;
    if ( hashtable->mode & HASH_OVERRIDE )
    {
	while ( slot )
	{
	    if ( ! (hashtable->cmp_fct)(slot->item, item) )
		return (slot->item) ;
	    slot = slot->next ;
	}
    }
/*
 * We must add it :
 */
    if ( ! (slot = (Hashslot) malloc (sizeof (HashslotRec))) )
    {
	perror ("HashtableAdd, malloc ") ;
	exit (1) ;
    }

    (hashtable->item_count)++ ;
    slot->item = item ;
    slot->next = *position ;
    *position  = slot ;
    return (slot) ;
}

void *HashtableRemove (hashtable, item)
    Hashtable hashtable ;
    void      *item ;

{
    register Hashslot slot, *pslot ;
    unsigned long     hashval ;

    hashval = (hashtable->hash_fct) (item) % hashtable->size ;
    slot  = hashtable->slots[hashval] ;
    pslot = &(hashtable->slots[hashval]) ;
    while ( slot )
    {
	if ( slot->item == item )
	{
	    (hashtable->item_count)-- ;
	    *pslot = slot->next ;
	    free (slot) ;
	    return (item) ;
	}
	pslot = &(slot->next) ;
	slot  = slot->next ;
    }

    return ((void *) 0) ;
}

/***************************************************************************
 * HashtableFind: retrieve the given entry
 *
 * If the hashtable is not in override mode, set its internal lookup
 * pointer to the found item, so that FindNext can proceed.
 *
 * RETURN: the entry in hashtable
 ***************************************************************************/

void *HashtableFind (hashtable, item)
    register Hashtable hashtable ;
    void               *item ;

{
    register Hashslot slot ;

    slot = hashtable->slots[(hashtable->hash_fct)(item) % hashtable->size] ;
    while ( slot )
    {
	if ( ! (hashtable->cmp_fct)(slot->item, item) )
	{
	    if ( ! (hashtable->mode & HASH_OVERRIDE) )
		hashtable->current_slot = slot ;
	    return (slot->item) ;
	}
	slot = slot->next ;
    }
    
    return ((void *) 0) ;
}

	
/***************************************************************************
 * HashtableFindNext: for HASH_OVERRIDE, find next item with given key
 ***************************************************************************/

void *HashtableFindNext (hashtable)
    Hashtable hashtable ;

{
    register Hashslot oslot, nslot ;

/*
 * This is only for debug purposes :
 */
    if ( (hashtable->mode & HASH_OVERRIDE) || ( ! hashtable->current_slot) )
    {
	fprintf (stderr, "HashtableFindNext: on OVERRIDE table ! (Fatal).\n") ;
	*((char *) 0) = 'C' ;	
    }
/*
 * Return next item, setting internal state :
 */
    oslot = hashtable->current_slot ;
    nslot = oslot->next ;
    while ( nslot )
    {
	if ( ! (hashtable->cmp_fct) (oslot->item, nslot->item) )
	{
	    hashtable->current_slot = nslot ;
	    return (nslot->item) ;
	}
	nslot = nslot->next ;
    }

    hashtable->current_slot = (Hashslot) 0 ;
    return ((void *) 0) ;
}

/***************************************************************************
 * HashtableMap: apply func & closure on all items in hashtable
 *
 * This function take cares of the fact that you mioght delete current 
 * slot in hook. This is the only case it handles...
 *
 * RETURN: # of applications.
 ***************************************************************************/

int HashtableMap (hashtable, func, closure)
    Hashtable hashtable ;
    void      (*func)() ;
    void      *closure ;

{
    register Hashslot *pslot, slot ;
    register int      count ;
    void	      *item ;

    for ( count = hashtable->item_count, pslot = hashtable->slots
	 ; count
	 ; pslot++ )
    {
	for ( slot = (*pslot) ; count && slot ; count-- )
	{
	    /*
	     * Preserve 'next' slot before hook, because the hook might destroy 
	     * current slot.
	     */
	    item = slot->item ;
	    slot = slot->next ;
	    (*func) (item, closure) ;
	}
    }

    return ( hashtable->item_count ) ;
}

/***************************************************************************
 * HashtableKeyedMap: map hashtable for items having key
 *
 * This function should be used to map specific items of a hashtable, it
 * provides a safe way to map the hashtable, even if it is called 
 * recursively (ie an HashtableKeyedMap can be called on the same hashtable
 * within a HashtableKeyedMap).
 * Only the current item can be detroyed in this function, otherwise
 * unpredicted things can happen (hum...a core dump !)
 *
 * RETURNS: # of mapped items
 ***************************************************************************/

int HashtableKeyedMap (hashtable, key, func, closure)
    Hashtable hashtable ;
    void      *key ;
    void      (*func)() ;
    void      *closure ;

{
    int               mapped = 0 ;
    register Hashslot slot, nslot ;
    
    slot  = hashtable->slots[(hashtable->hash_fct)(key) % hashtable->size] ;

    while ( slot )
    {
	nslot = slot->next ;

	if ( ! (hashtable->cmp_fct)(slot->item, key) )
	{
	    mapped++ ;
	    (func)(slot->item, closure) ;
	}

	slot = nslot ;
    }
    
    return (mapped) ;
}
